      subroutine vvsskk(u1,n1,p1,u2,n2,p2,u3,n3,p3,u,n,p,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a particle being a contraction of a quartic gauge-scalar boson 
c vertex with two polarization vectors and one scalar.
c
c Input: u1,u2 - vector arrays,
c        u3 - scalar array,
c        n1,n2,n3,n - numbers of polarization states,
c        p1,p2,p3 - momenta,
c        g - a quartic Higgs-gauge-boson coupling,
c        mass, wdth - mass and width of u.
c Output: u - scalar array,
c         p - its momentum.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 20 May, 2007 (based on vvhhkk.f); modified on 
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: n1,n2,n3,n,i1,i2,i3,i

      real(dp) :: mass,wdth
      real(dp), dimension(0:3) :: p1,p2,p3,p
      complex(dp) :: u1(0:3,n1),u2(0:3,n2),u3(1,n3),u(1,n),g,gden,mv2

      p=-p1-p2-p3

      mv2=cmplx(mass**2,-mass*wdth,dp)
      gden=g/(cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.d0,dp)-mv2)

      i=0

      do i3=1,n3
       do i2=1,n2
        do i1=1,n1
         i=i+1
         u(1,i)=-gden*u3(1,i3)
     &                *(u1(0,i1)*u2(0,i2)
     &                 -u1(1,i1)*u2(1,i2)
     &                 -u1(2,i1)*u2(2,i2)
     &                 -u1(3,i1)*u2(3,i2))
        end do
       end do
      end do

      end subroutine vvsskk
