      program carlocom
c=============================================================================c
c Calculates the total cross of a process whose matrix element has been
c generated with a program 'carlomat'
c
c                            Karol Kolodziej
c                         University of Silesia
c                            Katowice, Poland
c                        e-mail: karol.kolodziej@us.edu.pl
c=============================================================================c
 
      use mathprms
      use inprms 
      use partcls
      use drivekk
      use distribs
      use kincuts

      implicit none
 
      integer :: iborn,iscan0,nscan0,ie,j,ndim,ncall,neff0,
     &           ncut,ncut1,i,iopkch,ncalli,niter,n

      real(dp) :: crosskk,mtrsh,cs0,cssc(nkin),sd2,po,
     &            cs0i,var0i,sd0i,sumin,theta,dxmc,awmax,qw,
     &            cs0i2,cs02,csnum,csden,csacc,sdacc,chi2,chi2s,arg,val

      character(80) ind,ind1

      external crosskk

c Please supply the necessary information below by giving values to
c the parameters and variables

      integer, parameter :: ne=1  ! No of the cms energies

      real(dp), dimension(ne) ::  ! The cms energies:
     &         aecm=(/500.d0/)

c Calculate the Born cross section, iborn = 1(yes)/else(no), 
c in 'niter' iterations with 'ncalli' calls per iteration, i.e. 
c with 'niter x ncalli' calls;
c po is the fraction of old integration weights which should
c used in the next iterration, recomended value from 0.d0 to 0.5d0

      iborn=1
      niter=10
      ncalli=10000
      po=0.d0

c Generate the unweighted events: imc < 0(no)/else(yes)?
c If imc = 0 => use the maximum of the total cross section
c If imc = 1,2,... => use the maxima of the differential cross section No. imc.
c          See that idis=1 below, distribution imc is defined in calcdis.f
c          and teh corresponding test results in the total cross section

      imc=2

c Scan the Born cross section with 'nscan0' calls: iscan0 = 1(yes)/else(no)?

      iscan0=1
      nscan0=10

c Optimize the No of kinematical channels, iopkch=1(yes)/else(no)

      iopkch=0

c Discard kinematical channels which contribute less than qw of the dominant 
c channel in the initial scan

      qw=0.0d0

c Choose the scheme:
c     ischeme = 1(complex mass scheme)/else(fixed width scheme).

      ischeme=1

c Should distributions be calculated, idis=1(yes)/0(no)?

      idis=1

c Impose cuts, icuts=1(yes)/0(no)

      icuts=1

c If icuts=1 then select the cuts to be imposed

      if (icuts == 1) then
c on
c  cos theta(lepton,beam) - ictlb=1/0:
      ictlb=1
c      ctlb=0.985d0   ! 
      ctlb=cos(5.d0/180.d0*pi)
c  cos theta(quark,beam) - ictqb=1/0:
      ictqb=1
c      ctqb=0.985d0   ! 
      ctqb=cos(5.d0/180.d0*pi)
c  cos theta(gluon,beam) - ictgb=1/0:
      ictgb=0
c      ctgb=0.985d0   ! 
      ctgb=ctqb
c  cos theta(photon,beam) - ictab=1/0:
      ictab=1
c      ctab=0.985d0
      ctab=cos(5.d0/180.d0*pi)
c  cos theta(photon,lepton) - ictal=1/0:
      ictal=1
      ctal=cos(5.d0/180.d0*pi)
c  cos theta(photon,quark) - ictaq=1/0:
      ictaq=1
      ctaq=cos(5.d0/180.d0*pi)
c  cos theta(quark,quark') - ictqq=1/0:
      ictqq=1
c      ctqq=0.985d0   ! 
      ctqq=cos(10.d0/180.d0*pi)
c  cos theta(lepton,quark) - ictlq=1/0:
      ictlq=1
c      ctlq=0.985d0   ! 
      ctlq=cos(5.d0/180.d0*pi)
c  cos theta(lepton,gluon) - ictlg=1/0:
      ictlg=1
c      ctlg=0.985d0   ! 
      ctlg=ctlq
c  cos theta(lepton,lepton') - ictll=1/0:
      ictll=1
c      ctll=0.985d0   ! 
      ctll=cos(5.d0/180.d0*pi)
c  lepton energy (GeV):
      ienl=1
      ecutl=5.d0
c  missing transverse energy (GeV):
      iemt=1
      ecutt=15.d0
c  quark energy (GeV):
      ienq=1
      ecutq=15.d0
c  b-quark energy (GeV):
      ienb=0
      ecutb=15.d0
c  gluon energy (GeV):
      ieng=1
      ecutg=ecutq
c  photon energy (GeV):
      iena=1
      ecuta=1.d0
c  quark-quark' invariant mass (GeV):
      imqq=1
      mqq=10.d0
c  jet-jet' invariant mass (GeV):
      imjj=0
      mjj=10.d0
c  lepton-lepton' invariant mass (GeV):
      imll=0
      mll=10.d0
c  b-b' invarint mass (GeV):
      imbb=0
      mbb=20.d0

      end if ! icuts

c----------------------------------------------------------------------c
c End of input
c----------------------------------------------------------------------c

c Determine parameters of the proces:

      call parfixkk

      call logo

      mtrsh=zero
      sumin=zero

      do j=nisp+1,nepl
       mtrsh=mtrsh+m(j)
      end do
      do j=1,nisp
       sumin=sumin+m(j)
      end do


c The energy loop:

      do ie=1,ne

      ecm=aecm(ie)     !  The c.m.s. energy

      idisn=1 ! Characteristics of the distributions should be calculated anew

      if (ecm < sumin) then 
       write(*,*)'carlocom.f: The CMS energy below the mass of initial',
     &           ' state particles'
       write(*,*) sumin,' GeV'
       stop
      end if
      if (ecm < mtrsh) then 
       write(*,*)'carlocom.f: The CMS energy below the threshold',mtrsh,
     &           ' GeV'
       stop
      end if

      write(6,4000)
 4000 format('#-------------------------------------------------------',
     &       '----------------------#')
      write(6,1006) ecm
      write(6,4000)
 1006 format('#                       E_cm = ',f11.3,' GeV')

      if (icuts == 1) then

      if (ictlb == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'l') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,'(a)') '# Lepton-beam angle (deg):'
        theta=acos(ctlb)/pi*180.d0
        write(*,8050) theta,180.d0-theta,ctlb
 8050   format('# ',f10.4,' < Theta_lb <',f10.4,'  =>  |cos(Theta_lb)|',
     &  ' <',f12.8)
       end if
      end if

      if (ictqb == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'q') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,'(a)') '# Quark-beam angle (deg):'
        theta=acos(ctqb)/pi*180.d0
        write(*,8052) theta,180.d0-theta,ctqb
 8052   format('# ',f10.4,' < Theta_qb <',f10.4,'  =>  |cos(Theta_qb)|',
     &  ' <',f12.8)
       end if
      end if

      if (ictgb == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'g') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,'(a)') '# Gluon-beam angle (deg):'
        theta=acos(ctgb)/pi*180.d0
        write(*,8053) theta,180.d0-theta,ctgb
 8053   format('# ',f10.4,' < Theta_gb <',f10.4,'  =>  |cos(Theta_gb)|',
     &  ' <',f12.8)
       end if
      end if

      if (ictab == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'a') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,'(a)') '# Photon-beam angle (deg):'
        theta=acos(ctab)/pi*180.d0
        write(*,8054) theta,180.d0-theta,ctab
 8054   format('# ',f10.4,' < Theta_ab <',f10.4,'  =>  |cos(Theta_ab)|',
     &  ' <',f12.8)
       end if
      end if

      if (ictal == 1) then
       ncut=0
       ncut1=0
       do i=nisp+1,nepl
        if (expl(i) == 'a') then
         ncut=ncut+1
        else if (expl(i) == 'l') then
         ncut1=ncut1+1
        end if
       end do
       if (ncut*ncut1 > 0) then
        write(*,'(a)') '# Photon-lepton angle (deg):'
        theta=acos(ctal)/pi*180.d0
        write(*,8056) theta,ctal
 8056   format('# ',f10.4,' < Theta_al','  =>  cos(Theta_al) <',f12.8)
       end if
      end if

      if (ictaq == 1) then
       ncut=0
       ncut1=0
       do i=nisp+1,nepl
        if (expl(i) == 'a') then
         ncut=ncut+1
        else if (expl(i) == 'q') then
         ncut1=ncut1+1
        end if
       end do
       if (ncut*ncut1 > 0) then
        write(*,'(a)') '# Photon-quark angle (deg):'
        theta=acos(ctaq)/pi*180.d0
        write(*,8058) theta,ctaq
 8058   format('# ',f10.4,' < Theta_aq','  =>  cos(Theta_aq) <',f12.8)
       end if
      end if

      if (ictqq == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'q') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 1) then
        write(*,'(a)') '# Quark-quark'' angle (deg):'
        theta=acos(ctqq)/pi*180.d0
        write(*,8069) theta,ctqq
 8069   format('# ',f10.4,' < Theta_qq','  =>  cos(Theta_qq) <',f12.8)
       end if
      end if

      if (ictlq == 1) then
       ncut=0
       ncut1=0
       do i=nisp+1,nepl
        if (expl(i) == 'l') then
         ncut=ncut+1
        else if (expl(i) == 'q') then
         ncut1=ncut1+1
        end if
       end do
       if (ncut*ncut1 > 0) then
        write(*,'(a)') '# Lepton-quark angle (deg):'
        theta=acos(ctlq)/pi*180.d0
        write(*,8057) theta,ctlq
 8057   format('# ',f10.4,' < Theta_lq','  =>  cos(Theta_lq) <',f12.8)
       end if
      end if

      if (ictlg == 1) then
       ncut=0
       ncut1=0
       do i=nisp+1,nepl
        if (expl(i) == 'l') then
         ncut=ncut+1
        else if (expl(i) == 'g') then
         ncut1=ncut1+1
        end if
       end do
       if (ncut*ncut1 > 0) then
        write(*,'(a)') '# Lepton-gluon angle (deg):'
        theta=acos(ctlg)/pi*180.d0
        write(*,8158) theta,ctlg
 8158   format('# ',f10.4,' < Theta_lg','  =>  cos(Theta_lg) <',f12.8)
       end if
      end if

      if (ictll == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'l') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 1) then
        write(*,'(a)') '# Lepton-lepton'' angle (deg):'
        theta=acos(ctll)/pi*180.d0
        write(*,8059) theta,ctll
 8059   format('# ',f10.4,' < Theta_ll','  =>  cos(Theta_ll) <',f12.8)
       end if
      end if

      if (ienl == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'l') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,8230) ecutl
 8230   format('# Lepton energy:    E_l >',f10.4,' GeV')
       end if
      end if

      if (iemt == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'n') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,'(a,f10.4,a)')'# Missing transverse energy:   EmissT >',
     &        ecutt,' GeV'
       end if
      end if

      if (ienq == 1) then 
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'q') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,8232) ecutq
 8232   format('# Quark energy:    E_q >',f10.4,' GeV')
       end if
      end if

      if (ienb == 1) then 
       ncut=0
       do i=nisp+1,nepl
        if (expc(i)(1:1) == 'b') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,8332) ecutb
 8332   format('# b-quark energy:    E_b >',f10.4,' GeV')
       end if
      end if

      if (ieng == 1) then 
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'g') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,8233) ecutg
 8233   format('# Gluon energy:    E_g >',f10.4,' GeV')
       end if
      end if

      if (iena == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'a') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 0) then
        write(*,8234) ecuta
 8234   format('# Photon energy:    E_a >',f10.4,' GeV')
       end if
      end if

      if (imqq == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'q') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 1) then
        write(*,8400) mqq
 8400   format('# Quark-quark invariant mass:  m(q,q'') >',f10.4,
     &                                                      ' GeV')
       end if
      end if

      if (imbb == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expc(i)(1:1) == 'b') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 1) then
        write(*,8402) mbb
 8402   format('# bb'' invariant mass:  m(b,b'') >',f10.4,
     &                                                      ' GeV')
       end if
      end if

      if (imjj == 1) then
       ncut=0
       do i=nisp+1,nepl
        if (expl(i) == 'q' .or. expl(i) == 'g') then
         ncut=ncut+1
        end if
       end do
       if (ncut > 1) then
c        write(*,8405) mjj
c 8405   format('# Jet-jet invariant mass:  m(j,j'') >',f10.4,
c     &                                                      ' GeV')
        write(*,'(a,f10.4,a)') '# Jet-jet invariant mass:  m(j,j'') >',
     &       mjj,' GeV'
       end if
      end if

      if (imll == 1) then
       write(*,8410) mll
 8410  format('# Lepton-lepton invariant mass:  m(l,l'') >',f10.4,
     &                                                      ' GeV')
      end if

      write(6,4000)

      end if ! icuts


      if (iborn == 1) then
       write(6,5060) nepl-nisp
 5060 format('# The Born cross section with full ',i2,' particle',
     &       ' kinematics:')

c Initialize integation weights

      aw=aw0
      ndim=3*(nepl-nisp)-4

c Optimize the No of kinematical channels

      if (iopkch == 1) then
       call chanopt(ndim)
      end if

      if (imc >= 0) then
c Initialize maximum value of the cross section
       write(ind,'(f10.1)') ecm
       open(unit=11,file='mc_events.'//trim(adjustl(ind)),
     &                           status='replace',action='write')
       ieff=0
       imxtot=0
       imxdif=0
       itmax=0

       if (imc == 0) then
        crmax=0.d0
        open(unit=12,file='csmax_tot.'//trim(adjustl(ind)),
     &                           status='old',action='read',err=201)
c        read(12,'(e22.15)') crmax
        read(12,*) crmax
 201    close(12)
       else if (imc > 0) then
        cdmax=0.d0
        write(ind1,'(i16)') imc
        open(unit=12,file='csmax_'//trim(adjustl(ind1))//'dif.'//
     &       trim(adjustl(ind)),status='old',action='read',err=202)
        do i=1,nbmax
c         read(12,'(e22.15)') cdmax(i)
         read(12,*) cdmax(i)
        end do
 202    close(12)
       end if
      end if

      iscan=iscan0

      if (iscan == 1) then

c Scan the Born cross section in order to find the dominant channels:

        ncall=nscan0
        cs0=zero
        icall=0
        cssc=zero

        write(*,5110)
 5110 format('# Results of the initial scan:')
        write(*,5120)
 5120 format('#     Kinematics       Weight         Integral      ',
     &       'St.dev.       No of calls')

        sd2=zero

        do ikin=1,nkin
         if (aw(ikin) > 0.d0) then

          neff0=icall
          call carlos(crosskk,ndim,nkin,xmc,po,ncall,cs0i,var0i,sd0i)

          write(*,3002) ikin,aw(ikin),aw(ikin)*cs0i,aw(ikin)*sd0i,
     &                                                icall-neff0
 3002 format('#  ',i10,'     ',e15.6,e15.6,e12.3,i10)
          cs0=cs0+aw(ikin)*cs0i
          cssc(ikin)=aw(ikin)*cs0i
          sd2=sd2+(aw(ikin)*sd0i)**2
         end if
        end do

        write(*,4000)
        write(*,3800) icall,cs0,sqrt(sd2)
 3800 format('# Born scanned with',i8,' calls = ',e15.8,'+/-',e10.3)
        if (imc == 0) then
         write(*,3810) crmax
        end if
 3810 format('# The maximum value of the cross section= ',e15.8)
c        crmax=1.5d0*crmax
c        write(*,3820) crmax
c 3820 format('# shifted to ',e15.8)
        write(*,4000)

c Initialize integration weights:

        neff0=0
        xmc(0)=zero
        dxmc=zero
        awmax=zero
        do ikin=1,nkin
         if (aw(ikin) > zero) then
          aw(ikin)=cssc(ikin)/cs0
          if (aw(ikin) > awmax) awmax=aw(ikin)
         end if
        end do
        do ikin=1,nkin
         if (aw(ikin) > zero) then
          if (aw(ikin) < qw*awmax) then
           dxmc=dxmc+aw(ikin)
           aw(ikin)=zero
          end if
         end if
         xmc(ikin)=xmc(ikin-1)+aw(ikin)
        end do

        xmc=xmc/(1.d0-dxmc)

        iscan=0

        if (qw > 0.d0) then
         write(*,'(a,e9.2,a)') '# Kinematical channels contributing '
     &    //'less than',qw,' of the dominant channel'
         write(*,'(a)') '# discarded.'
        end if

      else ! iscan

c Initialize integration weights:

      neff0=0

      xmc(0)=zero

      do ikin=1,nkin
       xmc(ikin)=xmc(ikin-1)+aw(ikin)
      end do

      end if ! iscan

c The Born cross section with standard deviation:

c      ncall=ncall0
      ncall=niter*ncalli

      cs0=0.d0
      sd2=0
      icall=0

      if (idis == 1) then
       ncs=0
       di=zero
       ncsa=0
      end if

c      if (iisr == 1) then
c       write(*,3202)
c      else
       write(*,3200)
c      end if
 3200 format('# Calculating the Born cross section with the weights:')
c 3202 format('# Calculating the Born + ISR cross section with the',
c     &       ' weights:')

      csnum=0.d0
      csden=0.d0
      cs02=0.d0
      chi2s=0.d0

      do iit=1,niter
       write(*,4000)
       write(*,'(a,i3,a)') '# Iteration',iit,' with weights:'
       do ikin=1,nkin
        if (aw(ikin) > 0.d0) then
         write(*,'(a,i10, 2e16.8)') '#',ikin,aw(ikin),xmc(ikin)
        end if
       end do

       call carlos(crosskk,ndim,nkin,xmc,po,ncalli,cs0i,var0i,sd0i)

       cs0i2=cs0i*cs0i
       cs02=cs02+cs0i2
       csnum=csnum+cs0i*cs0i2/var0i
       csden=csden+cs0i2/var0i
       cs0=cs0+cs0i/niter
       chi2s=chi2s+cs0i2*(1.d0+1.d0/var0i)
       if (niter > 1) then
        sd2=sd2+(sd0i)**2/(niter*(niter-1))
       else
        sd2=(sd0i)**2
       end if

       csacc=csnum/csden
       sdacc=csden*iit/cs02
       chi2=sdacc*(chi2s/csden-csacc*csacc)/(iit-0.999d0)
       sdacc=sqrt(1.d0/abs(sdacc))

       write(*,4000)
       write(*,'(a,i3,a)') '# Iteration No. ',iit,':'
       write(6,2987) cs0i,sd0i,icall-neff0
       write(*,'(a,e13.6,a,e9.2,a,i9,a)') '# Accumulated result: ',
     &     csacc,' +/- ',sdacc,' with',icall,' calls'
c       write(*,'(a,e9.2)') '# Chi^2 per iterration: ',chi2
       if (imc == 0) then
        write(*,3810) crmax
       end if
       neff0=icall
       do ikin=1,nkin
        aw(ikin)=xmc(ikin)-xmc(ikin-1)
       end do
      end do

      write(*,4000)
      write(*,'(a)') '# Arithmetic mean result:'

      write(6,2987) cs0,sqrt(sd2),neff0

 2987 format('# Integral =',e13.6,' +/- ',e9.2,' with ',i9,' calls')

c      write(*,4000)
c      write(*,3030) dble(ieff)/dble(icall)
c 3030 format('# Efficiency of the event acceptance =',e10.3)

      if (idis == 1) then
       do j=1,ndist
        if (nbs(j) > 0) then
         write(*,4000)
         write(6,3000) j
 3000    format('# Distribution No. ',i3,':')
         write(6,'(a)')'# '//trim(desc(j))
         write(*,4000)

         arg=xmin(j)

         do n=1,nbs(j)

          if (ncs(n,j) > 0 .and. dx(n,j) > zero) then
c           val=di(n,j)/dx(n,j)
           val=di(n,j)/(dble(niter)*dx(n,j))
          else
           val=zero
          end if

c          if (imc == j) then
c           write(*,'(3e11.4,i4)') xdi(n-1,j),val,cdmax(n),itmax(n)
c           write(*,'(3e11.4,i4)') xdi(n,j),val,cdmax(n),itmax(n)
c          else
           write(*,'(2e11.4)') xdi(n-1,j),val
           write(*,'(2e11.4)') xdi(n,j),val
c          end if
         end do
        else
         exit
        end if
       end do

       write(*,4000)
       write(*,'(a)')'# Test of the distributions:'

       write(*,'(a,e15.8,a,i10,a)') '# Total cross section of ',cs0,
     &    ' with',neff0,' calls'

       write(*,'(a)') '# Integrated distributions:'

       do j=1,ndist
        if (nbs(j) > 0) then
         write(*,'(a,i3,a,e15.8,a,i10,a)') '#     No.',j,
     &                '  results in ',
     &                 sum(di(:,j))/niter,' with',sum(ncs(:,j)),' calls'
        else
         exit
        end if
       end do

      end if ! idis

      end if ! (iborn == 1)

      if (imc >= 0) then
       close(11)
       write(ind1,'(i16)') ieff
       write(ind,'(f10.1)') ecm
       write(*,4000)
       write(*,'(a)')'# '//trim(adjustl(ind1))//' MC events stored in '
     &     //'file '//'mc_events.'//trim(adjustl(ind))
       write(*,4000)

       if (imc == 0) then
        if (imxtot == 1) then
         open(unit=12,file='csmax_tot.'//trim(adjustl(ind)),
     &                           status='replace',action='write')
c         write(12,'(e22.15)') crmax
         write(12,*) crmax
         close(12)
        end if
       else if (imc > 0) then
        if (sum(ncs(:,imc)) /= neff0) then
         write(*,'(a,i3,a)') '# Distribution No. ',imc,' should not '//
     &    'have been used in the MC event generation.'
         write(*,'(a)') '# Generate MC events according to another '//
     &    'distribution, or change the limits'
         write(*,'(a)') '# within which the distribution is defined.'
         write(*,4000)
        end if
        if (imxdif == 1) then
         write(ind1,'(i16)') imc
         open(unit=12,file='csmax_'//trim(adjustl(ind1))//'dif.'//
     &        trim(adjustl(ind)),status='replace',action='write')
         do i=1,nbmax
c          write(12,'(e22.15)') cdmax(i)
          write(12,*) cdmax(i)
         end do
         close(12)
        end if
       end if
      end if

c      if (imc > 0) then
c       csmax=0.d0
c       do i=1,nbmax
c        if (ncs(i,imc) > 0) then
cc         csmax=csmax+cdmax(i)*dx(i,imc)*dble(ncsa(i))/dble(ncs(i,imc))
c         csmax=csmax+cdmax(i)*dble(ncsa(i))/dble(ncs(i,imc))
c        end if
c       end do
c       write(*,*)'Test:',csmax/nbs(imc)
c      else if (imc == 0) then
c       write(*,*)'Test:',crmax*ieff/neff0
c      end if

      end do ! ie

      end
